// Copyright (C) 1996 DIMACS Center, Rutgers, The State University of New Jersey
// Author(s): Patricia K. Fasel (Los Alamos Nat. Lab.)

// This software is copyrighted by the DIMACS Center at Rutgers, The State
// University of New Jersey.  IT IS PROVIDED AS IS, AND THE AUTHORS, DIMACS, AND
// RUTGERS, THE STATE UNIVERSITY OF NEW JERSEY  DISCLAIM
// ALL LIABILITY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY
// DERIVATIVES THEREOF, EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

// THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS SOFTWARE
// IS PROVIDED ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE
// NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
// MODIFICATIONS.

// The authors hereby grant permission to use, copy, modify, distribute,
// and license this software and its documentation for any purpose, provided
// that existing copyright notices are retained in all copies and that this
// notice is included verbatim in any distributions. No written agreement,
// license, or royalty fee is required for any of the authorized uses.
// Modifications to this software may be copyrighted by their authors
// and need not follow the licensing terms described here, provided that
// the new terms are clearly indicated on the first page of each file where
// they apply.

// Last File Update: 31-Jul-1996
// 

#ifndef general_h
#define general_h


// defines
#ifndef	FALSE
const int FALSE		= 0;
#endif

#ifndef TRUE
const int TRUE		= 1;
#endif


#ifndef Bool
typedef	int		Bool;
#endif


// all datatypes
enum	DataType      {	LIST = 0,	// singly linked list
			DLIST = 1,	// doubly linked list
			ARRAY = 2,	// dynamic array
			SORTARRAY = 3,	// sorted dynamic array
			BINARYHEAP = 4, // binary heap
			BINARYSEARCHTREE = 5,	// binary search tree
			REDBLACKTREE = 6,	// red black tree

			SETCOL = 10,	// unordered set
			MSETCOL = 11,	// unordered multi set
			SEQCOL = 12,	// sequence

			EDGEUND = 20,	// undirected edge
			EDGEDIR = 21,	// directed edge
			VERTEX = 22,	// vertex
			MIXEDHYPERGRAPH = 23,
			UNDHYPERGRAPH = 24,
			DIRHYPERGRAPH = 25,
			MIXEDBINARYGRAPH = 26,
			UNDBINARYGRAPH = 27,
			DIRBINARYGRAPH = 28,
			GRAPH = 29,
                        M_MIXEDHYPERGRAPH = 30,
                        M_UNDHYPERGRAPH = 31,
                        M_DIRHYPERGRAPH = 32,
                        M_MIXEDBINARYGRAPH = 33,
                        M_UNDBINARYGRAPH = 34,
                        M_DIRBINARYGRAPH = 35,

                        VERTEXGRAPHIC = 36,
                        EDGEGRAPHIC = 37,
                        BINEDGEGRAPHIC = 38,
                        HYPEREDGEGRAPHIC = 39,
			SORTEDLIST = 40 };

// direction of edge (moved to class Edge)
//enum    EdgeType        { UNDIRECTED = 0,
//                          DIRECTED = 1 };

// parameters
const int BUFSIZE	= 100;		// general buffer size
const int MAX_INT	= 2147483647;
const int LARGE_INT	= MAX_INT/2;
const int DIR_NAME_SZ	= 150;
const int HANDLE_NAME_SZ = 20;
const int ARRAYSIZE	= 100;		// grow size of dynamic arrays
const int BINHEAPSIZE	= 100;		// grow size of binary heaps
const int NAMESIZE	= 50;		// name size
const int COPY_ATTR	= 1;		// see loadGraphFromFile

//********************need a better color system
#define BLACK	((char*)"black")
#define BLUE	((char*) "blue")
#define GREEN	((char*)"green")
#define PURPLE	((char*)"purple")
#define BROWN	((char*)"brown")
#define ORANGE	((char*)"orange")
#define RED	((char*)"red")
#define GRAY	((char*)"gray")
#define YELLOW	((char*)"yellow")
#define WHITE	((char*) "white")

// types
typedef char*		String;		// general purpose string
typedef int		Flag;		// general purpose flag

enum ErrorType	{ LINK_OK,		// correct termination
		  LINK_ERR,		// error termination
		  LINK_GET_ATTR_FAIL,
		  LINK_SET_ATTR_FAIL,
		  LINK_NEW_ATTR_FAIL,
		  LINK_DELETE_ATTR_FAIL,
		  LINK_RESET_ATTR_FAIL,
		  LINK_FILEMISSING,
		  LINK_INDEXRANGE};

const String ErrorMessages[] = 
	{ "LINK successful",
	  "LINK error",
	  "Unable to get attribute",
	  "Unable to set attribute",
	  "Unable to make a new attribute",
	  "Unable to delete attribute",
	  "Unabel to reset attribute",
	  "File not found",
	  "Index out of range"};

// attribute parameters
enum    AttrType        { INTATTR=0,            // attribute is integer
                          FLOATATTR=1,          // attribute is float
                          STRINGATTR=2,         // attribute is string
                          OTHERATTR=3 };        // attribute is other
enum    AttrCategory    { ALLATTR=0,            // attribute for all objects
                          GRAPHATTR=1,          // attribute for all graphs
                          VERTEXATTR=2,         // attribute for all vertices
                          EDGEATTR=3,           // attribute for all edges
                          SELECTATTR=4 };       // attribute for selection
enum    AttrAction      { CREATEATTR=0,         // create an attribute
                          DELETEATTR=1,         // delete an attribute
                          EDITATTR=2,           // edit an attribute
                          DISPLAYATTR=3,        // display an attribute
                          RESETATTR=4 };        // reset an attribute

#ifndef DEFAULT_SET_IMPL
#define DEFAULT_SET_IMPL SortedList
#endif

#ifndef DEFAULT_SEQ_IMPL
#define DEFAULT_SEQ_IMPL  List
#endif

void warning(ErrorType);
void warning(String);

void error(ErrorType);
void error(String);

void fatalError(ErrorType);
void fatalError(String);

void Link_setSeed(long i);
void Link_initRand();
double Link_randomDouble0_1();
long Link_randomLong();


char *newString(char *src);

#endif
